/*********************************************************************
* Copyright (c) 2003-2007, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co_stream.c: Stream functions.
*
* $Id: co_stream.c,v 1.4 2007/05/08 19:25:25 CVSUSER Exp $
*
*********************************************************************/

#include "co.h"

#include <malloc.h>
#include <string.h>
#include <stdio.h>

#if 0
// debug on
#define IF_DBG(s) s
#else
#define IF_DBG(s)
#endif


co_stream co_stream_create(char * name, co_type datatype, int buffer_depth)
{
	co_stream stream = malloc(sizeof(co_stream_t));	
	if ( stream != NULL ) {
		stream->name = strdup(name);
		stream->datatype = datatype;
    stream->buffer_depth = buffer_depth;
		stream->direction = CO_NO_DIRECTION;
		stream->io_addr = 0;
	} else printf("malloc failed!");
	return stream;
}


int co_stream_eos(co_stream stream)
{
	// EOS not supported
	printf("WARNING:co_stream_eos() is not currently supported\n");  
	return 0;
}


void co_stream_attach(co_stream stream, int io_addr, co_stream_direction dir)
{
	stream->io_addr=io_addr;
	stream->direction=dir;
}

co_error co_stream_open(co_stream stream, mode_t mode, co_type type)
{
	int ret, num_bytes;
	int fd;
	//printf("opening stream... %s  %d\r\n", stream->name, stream->io_addr);

	switch (stream->io_addr) {
	case 0:
		fd = open(FSL_NODE_0, O_RDWR);
		fd0 = fd;
		break;
	case 1:
		fd = open(FSL_NODE_1, O_RDWR);
		fd1 = fd;
		break;
	case 2:
		fd = open(FSL_NODE_2, O_RDWR);
		fd2 = fd;
		break;
	case 3:
		fd = open(FSL_NODE_3, O_RDWR);
		fd3 = fd;
		break;
	default:
		printf("error: FSL number out of range!\n");
		break;
	}
	
	//printf("fd in co_stream_open: %d\r\n", fd);
	if (fd < 0) {
		perror("open");
		return 1;
	}
	
	num_bytes = type->width / 8;
	ret = ioctl(fd, FSLFIFO_DW_SET, num_bytes);
	if (ret < 0) {
		perror("dw_set");
		return -1;
	}
	ret = ioctl(fd, FSLFIFO_DW_GET, 0);
	if (ret != num_bytes) {
		fprintf(stderr, "Failed to set datawidth %d\n", num_bytes);
		return -1;
	}
}

int co_stream_close(co_stream stream) 
{

  return(0);
}

int co_stream_read(co_stream stream, void *buffer, int size)

{
	int len;

	switch (stream->io_addr) {
	case 0:
		fd = fd0;
		break;
	case 1:
		fd = fd1;
		break;
	case 2:
		fd = fd2;
		break;
	case 3:
		fd = fd3;
		break;
	default:
		printf("error: FSL number out of range!\n");
		break;
	}
	
	len = read(fd, buffer, size);
	//printf("stream_read... %d of bytes\r\n", len);
	if (len<=0) {
		fprintf(stderr, "Failed to read data\n");
		return -1;
	}
	else return 0;

}

// TO DO
int co_stream_read_nb(co_stream stream, void *buffer, int size)

{
#if 0
	int data,status;
	
	/* Return if empty or EOS. */
	if (((status=XIo_In32(stream->io_addr+8)&3))==0) return 0; 
	if ((status&2)!=0) return 0;

	while ( size > 4 ) {
		data=XIo_In32((stream)->io_addr);
		*(int *)buffer=data;
		((int *)buffer)++;
		size-=4;
	}
	data=XIo_In32((stream)->io_addr);
	switch (size) {
	case 1:
		*(char *)buffer=data;
		break;
	case 2:
		*(short *)buffer=data;
		break;
	case 3:
		*(short *)buffer=data;
		((short *)buffer)++;
		data >>= 16;
		*(char *)buffer=data;
		break;
	default:
		*(int *)buffer=data;
		break;
	}
#endif
	return 0;
}

int co_stream_write(co_stream stream, const void *buffer, int size)

{
	int len;

	switch (stream->io_addr) {
	case 0:
		fd = fd0;
		break;
	case 1:
		fd = fd1;
		break;
	case 2:
		fd = fd2;
		break;
	case 3:
		fd = fd3;
		break;
	default:
		printf("error: FSL number out of range!\n");
		break;
	}
	
	len = write(fd, buffer, size);
	//if (len != (FIFO_LEN * 1)) {
	//	fprintf(stderr, "Failed to write data\n");
	//	return -1;
	//}
	//printf("stream_write... %d of bytes\r\n", len);
	if (len <= 0){
		fprintf(stderr, "Failed to write data\n");
		return -1;
	}
	else {
		return 0;
	}
}

int co_stream_write_nb(co_stream stream, const void *buffer, int size)

{
#if 0
	int data;
	
	/* Return if full. */
	if ((XIo_In32(stream->io_addr+8)&1)==0) return 0;

	while ( size > 4 ) {
		data=*(int *)buffer;
		XIo_Out32((stream)->io_addr,data);
		((int *)buffer)++;
		size-=4;
	}
	switch (size) {
	case 1:
		data=*(char *)buffer;
		break;
	case 2:
		data=*(short *)buffer;
		break;
	default: 
		data=*(int *)buffer;
		break;
	}

	XIo_Out32((stream)->io_addr,data);
#endif
	return 1;
}

