##############################################################################
#
#     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
#     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
#     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
#     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
#     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
#     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
#     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
#     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
#     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
#     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
#     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
#     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#     FOR A PARTICULAR PURPOSE.
#
#     (c) Copyright 2004-2008 Xilinx, Inc.
#     All rights reserved.
#
##############################################################################


#
# Global MLD version
#
set MLD_version "linux_bc54_v1_00_a"


################################################################
#
# Globals
#
################################################################

#
# Counters for each peripheral type
#
array set instance_count {}

#
# EDK vs Linux side names
#
array set alias_table {}

#
# Output file names
#
set xparam_file_name "xparameters_bsp.h"
set kconfig_file_name "Kconfig.auto"

proc fix_path_cygwin {path} {
    set os_name [exec bash -c "uname -o"]
    if {[string compare -nocase $os_name "Cygwin"] == 0} {
	set path [exec bash -c "cygpath -m `cygpath -w $path`"]
    }
    return $path
}


################################################################
#
# Libgen interface
#
################################################################


proc linux_drc {lib_handle} {
    variable target_dir;

    puts "#--------------------------------------"
    puts "# Linux BSP DRC..."
    puts "#--------------------------------------"

    set bc_prefix [exec bash -c "echo \$BLUECAT_PREFIX"]

    set bc_prefix_xil [xget_value $lib_handle "PARAMETER" "BLUECAT_PREFIX"]
    set is_bluecat [exec bash -c "echo ${bc_prefix_xil}/*SETUP.sh"]

    if {[string compare -nocase $bc_prefix ""] == 0} {
	if {[string compare -nocase $bc_prefix_xil ""] == 0} {
	    error "Error: No target directory specified."
	}
	if {[string compare -nocase $is_bluecat "${bc_prefix_xil}/*SETUP.sh"] == 0 ||
	    [string compare -nocase $is_bluecat "${bc_prefix_xil}//*SETUP.sh"] == 0 } {
	    set target_dir "${bc_prefix_xil}"
	} else {
	    set target_dir "${bc_prefix_xil}/usr/src/linux"
	}
    } else {
	set target_dir "${bc_prefix}/usr/src/linux"
    }

    set target_dir [fix_path_cygwin $target_dir]

    if {[file isdirectory $target_dir] == 0} {
	error "Error: Target directory does not exist."
    }

    if {[file isdirectory $target_dir/include] == 0} {
	error "Error: Target directory is not a Linux kernel."
    }

    if {[file isdirectory $target_dir/arch] == 0} {
	error "Error: Target directory is not a Linux kernel."
    }

    puts "Using linux kernel: $target_dir"
}


proc generate {lib_handle} {

    puts "#--------------------------------------"
    puts "# Linux BSP generate..."
    puts "#--------------------------------------"

}


proc post_generate {lib_handle} {

    variable arch
    variable target_dir
    variable xparam_file_name
    variable kconfig_file_name

    set sw_proc_handle [xget_libgen_proc_handle]
    set drvlist [xget_handle $sw_proc_handle "DRIVER" "*"]
    set kernel_config [xget_value $lib_handle "PARAMETER" KERNEL_CONFIG]
    set os_name [exec bash -c "uname -o"]

    if {[string compare -nocase $os_name "Cygwin"] == 0} {
	set script_path [exec bash -c "cygpath -u `echo \$XILINX_EDK/../edk_user_repository/BlueCat/bsp/linux_bc54_v1_00_a/src/bin`"]
    } else {
	set script_path [exec bash -c "echo \$XILINX_EDK/../edk_user_repository/BlueCat/bsp/linux_bc54_v1_00_a/src/bin"]
    }

    set script_path [fix_path_cygwin $script_path]

    if {[file isdirectory $script_path] == 0} {
	set script_path "../../../bsp/linux_bc54_v1_00_a/src/bin"
    }

    if {[file isdirectory $script_path] == 0} {
	error "Error: MLD set must be either in the project directory or at the EDK level."
    }

    create_xparam
    create_kconfig
    add_xparam_cpu $sw_proc_handle
    add_xparam_drivers $drvlist

    puts "Linux target architecture is: ${arch}"
    puts "Updating the '$target_dir' Linux kernel to reflect the design"

    if {[file isdirectory ${target_dir}/include/asm-${arch}] == 0} {
	error "Error: This Linux kernel does not support the $arch architecture !"
    }

    if {[file isdirectory ${target_dir}/arch/${arch}] == 0} {
	error "Error: This Linux kernel does not support the $arch architecture !"
    }

    exec bash -c "cp -f $xparam_file_name ${target_dir}/include/asm-${arch}/xparameters_bsp.h"
    exec bash -c "cp -f $kconfig_file_name ${target_dir}/arch/${arch}/Kconfig.auto"

    if {$kernel_config != "none"} {
	if {$kernel_config == ""} {
	    set kernel_config "${target_dir}/.config"
	}
	set kernel_config [fix_path_cygwin $kernel_config]

        if {[file isfile $kernel_config] == 0} {
	    puts "kernel configuration file is not found !"
	    exit
        }
	puts "Updating the '$kernel_config' kernel configuration file to reflect the design"
   
	exec bash -c "mkdir -p /tmp"
	exec bash -c "cp -f  $script_path/update_config.sh /tmp"

	exec bash -c "cp -f $kconfig_file_name /tmp/"
	exec bash -c "chmod +x /tmp/update_config.sh"
	exec bash -c "/tmp/update_config.sh $kernel_config"
    }
}


################################################################
#
# Local utilities
#
################################################################


#
# Update configuration files with the CPU parameters.
#
proc add_xparam_cpu {proc_handle} {

    variable xparam_file_name
    variable arch

    set xparam_file [open $xparam_file_name a]

    # system clock
    set sys_clock [xget_sw_parameter_value $proc_handle "CORE_CLOCK_FREQ_HZ"]
    puts $xparam_file "\/* System Clock Frequency */"
    puts $xparam_file "#define XPAR_CPU_CLOCK_FREQ $sys_clock"
    puts $xparam_file ""

    # various CPU parameters
    set hwproc_handle [xget_handle $proc_handle "IPINST"]
    set args [xget_hw_parameter_handle $hwproc_handle "*"]

    set proctype [xget_value $hwproc_handle "OPTION" "IPNAME"]
    switch $proctype {
	"microblaze" {

	    set procver [xget_value $hwproc_handle "PARAMETER" "HW_VER"]
	    puts $xparam_file "\/* Processor ability to generate exceptions when in delay slot */"
	    if {[string compare -nocase $procver "5.00.a"] >= 0} {
		puts $xparam_file "#define XPAR_MICROBLAZE_0_EX_HANDLE_DELAY_SLOT 1"
	    } else {
		puts $xparam_file "#define XPAR_MICROBLAZE_0_EX_HANDLE_DELAY_SLOT 0"
	    }
	    puts $xparam_file ""

	    set proc_string "MICROBLAZE_0"

	    add_kconfig_microblaze $hwproc_handle
	    set arch "microblaze"
	}
	"ppc405" -
	"ppc405_virtex4" {
	    set proc_string "PPC405_0"
	    set arch "ppc"
	}
	"ppc440_virtex5" {
		set proc_string "PPC440_0"
		set arch "ppc"
	}
	default {
	    set proc_string "UNKNOWN"
	    puts "unsupported processor type $proctype\n"
	}
    }

    add_xparam_peripheral $xparam_file $proc_string $proctype $args
    puts $xparam_file ""

    close $xparam_file
}


#
# Update xparameters.h with the driver parameters.
#
proc add_xparam_drivers {drvlist} {
    
    variable instance_count
    variable alias_table
    variable xparam_file_name

    set xparam_file [open $xparam_file_name a]
    
    # clean-up the global data
    array unset instance_count
    array set instance_count {}
    array unset alias_table
    array set alias_table {}

    foreach drv $drvlist {
	add_xparam_driver $xparam_file $drv
    }

    puts $xparam_file "\/* Interrupt settings */"
    foreach drv $drvlist {
	add_xparam_interrupts $xparam_file $drv
    }
    puts $xparam_file ""

    puts $xparam_file "\/* Peripheral counts */"
    foreach p [array names instance_count] {
	set format_str "#define XPAR_%s_NUM_INSTANCES %i"
	set all_str [format $format_str $p $instance_count($p)]
	puts $xparam_file $all_str
    }
    puts $xparam_file ""

    puts $xparam_file "\/* LocalLink parameters *\/"
    close $xparam_file
    catch {exec grep LLINK_CONNECTED ../../include/xparameters.h >> $xparam_file_name}
    catch {exec grep XPAR_XLLDMA_USE_DCR ../../include/xparameters.h >> $xparam_file_name}
}


#
# Get the port name.
#
proc get_port_val {ports port} {

    foreach cur_port $ports {
	set port_name [string toupper [xget_value $cur_port "NAME"]]
	set port_name [string map -nocase {C_ ""} $port_name]

	if {[string compare -nocase $port_name $port] == 0} {
	    set port_value [string toupper [xget_value $cur_port "VALUE"]]
	    return $port_value
	  }
	}

	return absent
}


#
# Strip any opb_ plb_ mch_ or dcr_ prefixes from param name.
# A number of iterations needed.
#
proc strip_bus {periph_name} {

	regsub -all -nocase OPB_ $periph_name "" periph_name
	regsub -all -nocase PLB_ $periph_name "" periph_name
	regsub -all -nocase DCR_ $periph_name "" periph_name
	regsub -all -nocase MCH_ $periph_name "" periph_name
	regsub -all -nocase XPS_ $periph_name "" periph_name
	regsub -all -nocase MPMC_ $periph_name "" periph_name
	
	return $periph_name
}

proc normalize_C_def {definition} {

	regsub -all {\.} $definition "_" definition
	set definition [string toupper $definition]
	
	return $definition
}



#
# Get the final hardware connected to this IP core and provide unified
# names for IP cores.
#
proc get_hardware_name {periph_name ports} {

    set periph_name [strip_bus $periph_name]

    switch $periph_name {
	EMC {
	    set hw [get_port_val $ports "Mem_A"]
	    if {[string compare -nocase $hw "fpga_0_FLASH_2Mx32_Mem_A_split"] == 0} {
		#case of ML403 - ML403_FLASH
		return ML403_FLASH
	    }
	    return EMC_FLASH
	}

	ETHERNET {
	    # Xilinx changed name for EMAC core in latest EDK, so we should fix it
	    return EMAC
	}
    }

    return $periph_name	
}

#
# Update xparameters.h with parameters for a particular driver.
#
proc add_xparam_driver {xparam_file drv_handle} {

    set periphs [xget_sw_iplist_for_driver $drv_handle]
    set drv_params [xget_handle $drv_handle "PARAMETER" "*"]

    # global array to keep count of each peripheral type
    variable instance_count
    variable alias_table

    # general driver parameters
    set drv_name [xget_value $drv_handle "NAME"]
    puts $xparam_file ""
    puts $xparam_file [format "/* Driver parameters for '%s' */" $drv_name]
    set skip_first 1

    foreach drv_param $drv_params {
	if {$skip_first} {
	    set skip_first 0
	    continue
	}
	set drv_param_name [xget_value $drv_param "NAME"]
	set drv_param_val [xget_value $drv_handle "PARAMETER" $drv_param_name]
	set macro [string toupper [format "%s_%s" $drv_name $drv_param_name]]	
	puts $xparam_file "#define $macro ${drv_param_val}"
    }	
    puts $xparam_file ""

    # ip cores parameters
    foreach periph $periphs {
	set args [xget_hw_parameter_handle $periph "*"]
	set ports [xget_hw_port_handle $periph "*"]
	set periph_name [string toupper [xget_value $periph "VALUE"]]

	# get real hardware connected to this IP core (e.g.  is EMC IP core connected to FLASH or to USB ?)
	set hw_periph_name [get_hardware_name $periph_name $ports]

	# initialize / update count for this peripheral type
	if {![string compare -nocase "" [array names instance_count $hw_periph_name]]} {
	    set instance_count($hw_periph_name) 1
	} else {
	    incr instance_count($hw_periph_name)
	}

	set device_id [expr $instance_count($hw_periph_name)-1];

	if {!$device_id} {
	    add_kconfig_hardware $hw_periph_name
	}

	# Output standard core params
	set periph_string [format "%s_%s" $hw_periph_name $device_id]
	add_xparam_peripheral $xparam_file $periph_string $periph_name $args

	set periph_alias [xget_value $periph "NAME"]
	set alias_table($periph_alias) $periph_string

	puts $xparam_file ""
    }
    puts $xparam_file ""
}


#
# Update xparameters.h with parameters for a particular peripheral instance.
#
proc add_xparam_peripheral {xparam_file periph_name hw_periph_name params} {

    puts $xparam_file "\/* Definitions for $periph_name */"
    set instance_found 0 

    foreach param $params {

	# Avoid double occurrence of the INSTANCE parameter
	set param_name [string toupper [xget_value $param "NAME"]]
	regsub -nocase ^C_ $param_name "" param_name 
	if {[string compare -nocase $param_name "instance"] == 0} {
	    if {$instance_found == 1} {
		continue
	    }
	    set instance_found 1
	}
	
	if {[string compare -nocase $param_name "HW_VER"] == 0} {
		set hw_periph_ver [xget_value $param "VALUE"]
	}
	

	set param_name [format "XPAR_%s_%s" $periph_name $param_name]
	set param_value [xget_value $param "VALUE"]
	set data_type [string toupper [xget_hw_subproperty_value $param "DT"]]
	
	switch -regexp $data_type {
	    INTEGER {
		set param_value [string map {_ ""} $param_value]
	    }
	    STD_LOGIC_VECTOR* {
		set param_value [string map {_ ""} $param_value]
		if {[string match -nocase 0b* $param_value]} {
		    set param_value [xconvert_binary_to_hex $param_value]
		} elseif {[string match -nocase 0x* $param_value]} {
		    set param_value [format "%s" $param_value]
		} else {
		    set param_value [xconvert_binary_to_hex 0b$param_value]
		}
	    }
	    STD_LOGIC {
	    }
	    default {
		set param_value "\"${param_value}\""
	    }
	}

	puts $xparam_file "#define $param_name $param_value"
    }
# and add ip core name which is not
    set param_name [format "XPAR_%s_%s" $periph_name "HW_NAME"]
    set hw_periph_name [string tolower $hw_periph_name]
    puts $xparam_file "#define $param_name \"$hw_periph_name\""
# composite ip core definition
    set hw_composite [format "%s_%s" $hw_periph_name $hw_periph_ver]
    set hw_composite [normalize_C_def $hw_composite]
    puts $xparam_file "#define $hw_composite"
}


#
# Update xparameters.h with the interrupt info.
#
proc add_xparam_interrupts {xparam_file drv_handle} {

    variable instance_count
    variable alias_table
    lappend irq_list

    set periphs [xget_sw_iplist_for_driver $drv_handle]

    foreach periph $periphs {

	set periph_name [string toupper [xget_value $periph "VALUE"]]
	set ports [xget_hw_port_handle $periph "*"]	

	if {[string match -nocase *_intc $periph_name]} {

	    set intc_alias [xget_value $periph "NAME"]
	    set source_ports [xget_interrupt_sources $periph]
	    set intc_name $alias_table($intc_alias)

	    set i 0
	    foreach source_port $source_ports {
		if {[llength $source_port] == 0} {
		    incr i
		    continue
		}
		set source_periph [xget_handle $source_port "PARENT"]
		set source_alias [xget_value $source_periph "NAME"]

		if {![string compare -nocase "" [array names alias_table $source_alias]]} {
		    set name_prefix [string toupper $source_alias]
		    if {![string compare -nocase "" [array names instance_count $name_prefix]]} {
			set instance_count($name_prefix) 1
		    } else {
			incr instance_count($name_prefix)
		    }
		    set device_id [expr $instance_count($name_prefix)-1];
		    set periph_string [format "%s_%s" $name_prefix $device_id]
		    set alias_table($source_alias) $periph_string
		}

		set source_name $alias_table($source_alias)
		
		# get real hardware connected to this IP core (e.g. is EMC IP core connected to FLASH or to USB ?)
		set hw_periph_name [get_hardware_name $source_name $ports]
		set source_name $hw_periph_name
		
		set irq_string [format "%s_IRQ" $source_name]
		if {[lsearch $irq_list $irq_string] >= 0} {
		    puts "Warning: ignoring redefinition of ${irq_string}"
		    incr i
		    continue
		}
		lappend irq_list $irq_string

		puts $xparam_file "#define XPAR_$irq_string $i"
		incr i
	    }
	}
    }
}


#
# Create xparameters.h
#
proc create_xparam {} {

    variable xparam_file_name

    set xparam_file [open $xparam_file_name w]

    puts $xparam_file "/********************************************************************"
    puts $xparam_file " *"
    puts $xparam_file " * CAUTION: This file is automatically generated by libgen."
    puts $xparam_file " * Version: [xget_swverandbld]"
    puts $xparam_file " *"
    puts $xparam_file " * Copyright (c) 2007 Xilinx, Inc.  All rights reserved."
    puts $xparam_file " *"
    puts $xparam_file " * Description: BlueCat Linux 5.4 Driver Parameters"
    puts $xparam_file " *"
    puts $xparam_file " ********************************************************************/"
    puts $xparam_file ""

    close $xparam_file
}


#
# Create kernel configuration.
#
proc create_kconfig {} {

    variable kconfig_file_name

    set kconfig_file [open $kconfig_file_name w]

    puts $kconfig_file "\#"
    puts $kconfig_file "\# CAUTION: This file is automatically generated by libgen."
    puts $kconfig_file "\# Version: [xget_swverandbld]"
    puts $kconfig_file "\# DO NOT EDIT."
    puts $kconfig_file "\#"
    puts $kconfig_file "\# Copyright (c) 2007 Xilinx, Inc.  All rights reserved."
    puts $kconfig_file "\#"
    puts $kconfig_file "\# Description: BlueCat Linux 5.4 Kernel Configuration"
    puts $kconfig_file "\#"
    puts $kconfig_file ""

    puts $kconfig_file "config XILINX_EDK"
    puts $kconfig_file "\tbool"
    puts $kconfig_file "\tdefault y"
    puts $kconfig_file ""

    close $kconfig_file
}


#
# Update the kernel configuration with the CPU parameters.
#
proc add_kconfig_microblaze {proc_handle} {

    variable kconfig_file_name
    lappend mb_opt USE_BARREL USE_FPU USE_PCMP_INSTR USE_DIV USE_HW_MUL USE_MMU

    set kconfig_file [open $kconfig_file_name a]

    foreach opt $mb_opt {
	set cnf "MICROBLAZE_${opt}"
	set val [xget_value $proc_handle "PARAMETER" "C_${opt}"]
	
	puts $kconfig_file "config MICROBLAZE_${opt}"
	puts $kconfig_file "\thex"
	puts $kconfig_file "\tdefault $val"
	puts $kconfig_file ""
    }

    set procver [xget_value $proc_handle "PARAMETER" "HW_VER"]
    puts $kconfig_file "config MICROBLAZE_VERSION"
    puts $kconfig_file "\tstring"
    puts $kconfig_file "\tdefault \"v${procver}\""
    puts $kconfig_file ""

    close $kconfig_file

    set val [xget_value $proc_handle "PARAMETER" "C_FSL_LINKS"]
    if {$val > 0} {
	add_kconfig_hardware "FSL_LINKS"
    }
}


#
# Add supported drivers to the kernel configuration.
#
proc add_kconfig_hardware {hardware} {

    variable kconfig_file_name

    set kconfig_file [open $kconfig_file_name a]

	puts $kconfig_file "config HAVE_XILINX_${hardware}"
	puts $kconfig_file "\tbool"
	puts $kconfig_file "\tdefault y"
	puts $kconfig_file ""

    close $kconfig_file
}
